* Output options: all can be turned on/off by adding or removing the NO;
* page number, date, centering, or page breaks, page length and width;
OPTIONS NOnumber NOdate NOcenter FormDlim=" " PageSize=MAX LineSize=MAX SPOOL;
* Eliminate SAS default titles and names of tables in output (TRACE ON to show);
TITLE; ODS TRACE OFF; *ODS LISTING;

***********************************************************************************;
*******     MACRO PROGRAMS TO AUTOMATE CALCULATIONS FOR THIS EXAMPLE        *******;
*******               NOTHING IN HERE NEEDS TO BE CHANGED                   *******;
***********************************************************************************;

* To use TotalR2 macro;
* DV =        Case-sensitive name of dependent variable (required);
* PredFewer = Name of OUTPM= data file of predicted outcomes for first model (required);
* PredMore =  Name of OUTPM= data file of predicted outcomes for second model (optional);
%MACRO TotalR2(DV=, Pred1=, Pred2=);
PROC CORR NOPRINT NOSIMPLE DATA=&Pred1. OUTP=Corr1; VAR pred &DV.; RUN;
DATA Corr1; LENGTH Name $30.; SET Corr1; Name="&Pred1.";
     IF Pred=. THEN Pred=0; PredCorr=Pred; TotalR2=PredCorr*PredCorr; 
     IF _NAME_="Pred" OR MISSING(_NAME_)=1 THEN DELETE; 
     KEEP Name PredCorr TotalR2; RUN;
%IF %LENGTH(&Pred2.)>0 %THEN %DO;
PROC CORR NOPRINT NOSIMPLE DATA=&Pred2. OUTP=Corr2; VAR pred &DV.; RUN;
DATA Corr2; LENGTH Name $30.; SET Corr2; Name="&Pred2."; 
     IF Pred=. THEN Pred=0; PredCorr=Pred; TotalR2=PredCorr*PredCorr; 
     IF _NAME_="Pred" OR MISSING(_NAME_)=1 THEN DELETE; 
     KEEP Name PredCorr TotalR2; RUN;
DATA CorrCompare; LENGTH Name $30.; SET Corr1 Corr2; 
     TotalR2Diff=TotalR2-LAG1(TotalR2);
     KEEP Name PredCorr TotalR2 TotalR2Diff; RUN; 
%END;
%IF %LENGTH(&Pred2.)>0 %THEN %LET ToPrint=CorrCompare; 
     %ELSE %LET ToPrint=Corr1;
TITLE8 "Total-R2 Per Model";
%IF %LENGTH(&Pred2.)>0 %THEN %DO;
TITLE9 "Change in Total-R2 for &Pred1. vs. &Pred2."; %END;
PROC PRINT NOOBS DATA=&ToPrint.; RUN; TITLE8; TITLE9;   
%MEND TotalR2;

* To use PseudoR2 macro;
* Ncov =     TOTAL # entries in covariance parameter estimates table;
* CovFewer = Name of ODS CovParms table for nested baseline model (required);
* CovMore1 = Name of ODS CovParms table for comparison model1 (required);
* CovMore2 = Name of ODS CovParms table for comparison model2 (optional);
%MACRO PseudoR2(NCov=, CovFewer=, CovMore1=, CovMore2=);
DATA &CovFewer.; LENGTH Name $30.; SET &CovFewer.; Name="&CovFewer."; RUN;
DATA &CovMore1.; LENGTH Name $30.; SET &CovMore1.; Name="&CovMore1."; RUN;
DATA CovCompare1; LENGTH Name $30.; SET &CovFewer. &CovMore1.; RUN;
DATA CovCompare1; SET CovCompare1; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
%IF %LENGTH(&CovMore2.)>0 %THEN %DO; %LET lagchange=%EVAL(&Ncov.*2);
DATA &CovMore2.; LENGTH Name $30.; SET &CovMore2.; Name="&CovMore2."; RUN;
DATA CovCompare2; LENGTH Name $30.; SET &CovFewer. &CovMore2.; RUN;
DATA CovCompare2; SET CovCompare2; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
DATA CovCompare; SET CovCompare1 CovCompare2;
     PseudoR2Change=PseudoR2-(LAG&lagchange(PseudoR2)); RUN;
%END; %IF %LENGTH(&CovMore2.)>0 %THEN %LET ToPrint=CovCompare; 
     %ELSE %LET ToPrint=CovCompare1;
DATA &ToPrint.; SET &ToPrint.; 
     IF CovParm IN("UN(2,1)","UN(3,1)","UN(3,2)","UN(4,1)","UN(4,2)","UN(4,3)",
                   "UN(5,1)","UN(5,2)","UN(5,3)","UN(5,4)",
                   "UN(6,1)","UN(6,2)","UN(6,3)","UN(6,4)","UN(6,5)") THEN DELETE; 
     DROP Zvalue ProbZ; RUN;
TITLE8 "Pseudo-R2 Relative to &CovFewer.";
%IF %LENGTH(&CovMore2.)>0 %THEN %DO;
TITLE9 "Change in Pseudo-R2 for &CovMore1. vs. &CovMore2."; %END;
PROC PRINT NOOBS DATA=&ToPrint.; RUN; TITLE8; TITLE9;
%MEND PseudoR2;

* To use FitTest macro;
* FitFewer = name of infocrit table for nested model;
* FitMore  = name of infocrit table for comparison model;
%MACRO FitTest(FitFewer=, FitMore=);
DATA &FitFewer.; LENGTH Name $30.; SET &FitFewer.; Name="&FitFewer."; RUN;
DATA &FitMore.;  LENGTH Name $30.; SET &FitMore.;  Name="&FitMore.";  RUN;
DATA FitCompare; LENGTH Name $30.; SET &FitFewer. &FitMore.; RUN;
DATA FitCompare; SET FitCompare; DevDiff=Lag1(Neg2LogLike)-Neg2LogLike;
     DFdiff=Parms-LAG1(Parms); Pvalue=1-PROBCHI(DevDiff,DFdiff);
     DROP AICC HQIC CAIC; RUN;
TITLE2 "Likelihood Ratio Test for &FitFewer. vs. &FitMore.";
PROC PRINT NOOBS DATA=FitCompare; RUN; TITLE2;
%MEND FitTest;


***********************************************************************************;
*******           BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 4           *******;
*******               CHANGE "filesave" TO YOUR FOLDER PATH                 *******;
***********************************************************************************;

* Define global variable for file location to be replaced in code below;
* \\Client\ precedes actual path when using UIowa Virtual Desktop;
%LET filesave=C:\Dropbox\23_PSQF6272\PSQF6272_Example4;

* Import trimmed example data excel data file from sheet "grade10" into SAS;
PROC IMPORT DATAFILE="&filesave.\Example4_Data.xlsx" OUT=work.Example4 DBMS=XLSX REPLACE; 
     SHEET="grade10"; GETNAMES=YES; RUN;

* Open external file to save results to;
ODS RTF FILE="&filesave.\Example4_SAS_Output.rtf" 
        STYLE=HTMLBlue STARTPAGE=NO BODYTITLE;

TITLE "SAS Descriptive Statistics within Student-Level Data";
PROC FREQ DATA=work.Example4; TABLE lunch; RUN;

* Data preparation;
DATA work.Example4; SET work.Example4; 
* Add labels to original variables;
  LABEL districtID= "districtID: District ID number"
        studentID=  "studentID: Student ID number"
        schoolID=   "schoolID: School ID number"
        lunch=      "lunch: 0=Paid, 1=Reduced, 2=Free"
        math=       "math: Math Test Score";
* Create new dummy-coded predictors; 
       IF lunch=0 THEN DO; PvRF=0; PRvF=0; lunch0=1; lunch1=0; lunch2=0; END;
  ELSE IF lunch=1 THEN DO; PvRF=1; PRvF=0; lunch0=0; lunch1=1; lunch2=0; END;
  ELSE IF lunch=2 THEN DO; PvRF=1; PRvF=1; lunch0=0; lunch1=0; lunch2=1; END;
  LABEL PvRF=   "PvRF: 0=Paid, 1=Reduced or Free Lunch"
        PRvF=   "PRvF: 0=Paid or Reduced, 1=Free Lunch"
	   lunch0= "lunch0: 1=Paid Lunch"
        lunch1= "lunch1: 1=Reduced Lunch"
        lunch2= "lunch2: 1=Free Lunch"; 
* Filter to complete cases before computing cluster means;
  IF NMISS(math,lunch)>0 THEN DELETE;
RUN;

* Create cluster means;
PROC SORT DATA=work.Example4; BY schoolID; RUN;
PROC MEANS NOPRINT DATA=work.Example4; BY schoolID;
     VAR math PvRF PRvF;
     OUTPUT OUT=work.SchoolMeans 
          MEAN(math PvRF PRvF lunch0 lunch1 lunch2)= 
               CM_math CM_PvRF CM_PRvF CM_lunch0 CM_lunch1 CM_lunch2; 
RUN;

* Label new cluster means;
DATA work.SchoolMeans; SET work.SchoolMeans;
     schoolN=_FREQ_; DROP _TYPE_ _FREQ_;
     LABEL schoolN=   "schoolN: # Students Sampled Per School"
           CM_math=   "CM_math: School Mean Math Test Score"
           CM_PvRF=   "CM_PvRF: School Mean 0=No, 1=Reduced or Free Lunch"
           CM_PRvF=   "CM_PRvF: School Mean 0=Paid or Reduced, 1=Free Lunch"
           CM_lunch0= "CM_lunch0: School Mean Paid Lunch" 
           CM_lunch1= "CM_lunch1: School Mean Reduced Lunch"
           CM_lunch2= "CM_lunch2: School Mean Free Lunch";
RUN;

TITLE "SAS Descriptive Statistics within School-Level Data";
PROC MEANS NDEC=2 DATA=work.SchoolMeans; 
     VAR schoolN CM_math CM_PvRF CM_PRvF CM_lunch0 CM_lunch1 CM_lunch2; RUN;
TITLE;

* Merge school means back into student data and center predictors;
DATA work.Example4; MERGE work.Example4 work.SchoolMeans; BY schoolID; 
* Center cluster means so ref school has 0=.70, 1=.11, and 2=.19;
  CM_PvRF30 = CM_PvRF - .30;
  CM_PRvF19 = CM_PRvF - .19;  
* Cluster-mean-center level-1 predictors for random slopes;
  WC_PvRF = PvRF - CM_PvRF;
  WC_PRvF = PRvF - CM_PRvF;
LABEL WC_PvRF= "WC_PvRF: Within-Cluster Paid vs Reduced/Free Lunch"
      WC_PRvF= "WC_PRvF: Within-Cluster Paid/Reduced vs Free Lunch";
RUN;

TITLE "SAS Descriptive Statistics within Student-Level Data";
PROC MEANS NOLABELS NDEC=3 N MEAN VAR STDDEV MIN MAX DATA=work.Example4; 
	VAR WC_PvRF WC_PRvF; RUN;
TITLE;

***********************************************************************************;
*******                    BEGIN MODELS FOR EXAMPLE 4                       *******;
***********************************************************************************;

* Empty Models;

TITLE "SAS Model 1: Empty Means, Random Intercept for Math";
PROC MIXED DATA=work.Example4 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID; 
     MODEL math = / SOLUTION DDFM=Satterthwaite; 
     RANDOM INTERCEPT / VCORR TYPE=UN SUBJECT=schoolID; * VCORR gives ICC;
     ODS OUTPUT CovParms=CovEmpty; * Save variances; 
RUN; TITLE;

TITLE "SAS: Empty Means, Random Intercept for Ordinal Student Lunch";
PROC GLIMMIX DATA=work.Example4 NOCLPRINT NAMELEN=100 GRADIENT METHOD=QUAD(QPOINTS=7);
     CLASS schoolID; 
     MODEL lunch (DESCENDING) = / SOLUTION LINK=CLOGIT DIST=MULT DDFM=NONE; 
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID; 
     COVTEST "Need Random Intercept?" ZEROG;
RUN; TITLE;


***********************************************************************************;
* Add Paid vs Reduced/Free Lunch predictor slopes;

TITLE "SAS Model 2a: Add L1 Binary Student Paid vs Reduced/Free Lunch";
PROC MIXED DATA=work.Example4 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID;         
     MODEL math = PvRF / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID; 
     ODS OUTPUT CovParms=CovSmush; * Save variances; 
RUN; TITLE;
* Pseudo-R2 relative to empty means, random intercept model;
%PseudoR2(NCov=2, CovFewer=CovEmpty, CovMore1=CovSmush);

TITLE "SAS Model 2b: Add L2 Cluster Mean of Paid vs Reduced/Free Lunch";
PROC MIXED DATA=work.Example4 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID;         
     MODEL math = PvRF CM_PvRF30 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID; 
     ODS OUTPUT CovParms=CovContext InfoCrit=FitContext; * Save variances and fit; 
     ESTIMATE "L2 PvRF Between Slope"  PvRF 1 CM_PvRF30 1;
RUN; TITLE;
* Pseudo-R2 relative to empty means, random intercept model;
%PseudoR2(NCov=2, CovFewer=CovEmpty, CovMore1=CovSmush, CovMore2=CovContext);

TITLE "SAS Model 2c: Add Random Slope for Cluster-MC Paid vs Reduced/Free Lunch";
PROC MIXED DATA=work.Example4 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID;         
     MODEL math = PvRF CM_PvRF30 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT WC_PvRF / GCORR TYPE=UN SUBJECT=schoolID; 
     ODS OUTPUT CovParms=CovRand InfoCrit=FitRand; * Save variances; 
RUN; TITLE;
* LRT for random slope;
%FitTest(FitFewer=FitContext, FitMore=FitRand);

TITLE "SAS Model 3a: Add Intra-Variable Cross-Level Interaction";
PROC MIXED DATA=work.Example4 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID;         
     MODEL math = PvRF CM_PvRF30 PvRF*CM_PvRF30 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT WC_PvRF / GCORR TYPE=UN SUBJECT=schoolID; 
     ODS OUTPUT CovParms=Cov3a; * Save variances; 
RUN; TITLE;
* Pseudo-R2 relative to random slope model (new baseline);
%PseudoR2(NCov=4, CovFewer=CovRand, CovMore1=Cov3a);

TITLE "SAS Model 3b: Add Level-2 Quadratic Interaction";
PROC MIXED DATA=work.Example4 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID;         
     MODEL math = PvRF CM_PvRF30 PvRF*CM_PvRF30 CM_PvRF30*CM_PvRF30 
                   / SOLUTION DDFM=Satterthwaite OUTPM=Pred3b;
     RANDOM INTERCEPT WC_PvRF / GCORR TYPE=UN SUBJECT=schoolID; 
     ODS OUTPUT CovParms=Cov3b; * Save variances; 
     ESTIMATE "Simple L2 PvRF Between Slope"  PvRF 1 CM_PvRF30 1;
     ESTIMATE "L2 PvRF Between*CM_PvRF30"     PvRF*CM_PvRF30 1 CM_PvRF30*CM_PvRF30 1;
RUN; TITLE;
* Pseudo-R2 relative to random slope model (new baseline);
%PseudoR2(NCov=4, CovFewer=CovRand, CovMore1=Cov3a, CovMore2=Cov3b);
* Total-R2 relative to empty means model;
%TotalR2(DV=math, Pred1=Pred3b);

***********************************************************************************;
* Add Paid/Reduced vs Free Lunch predictor slopes;

TITLE "SAS Model 4a: Add 2 Main Effect Slopes for Paid/Reduced vs Free Lunch";
PROC MIXED DATA=work.Example4 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID;         
     MODEL math = PvRF CM_PvRF30 PvRF*CM_PvRF30 CM_PvRF30*CM_PvRF30 
                  PRvF CM_PRvF19 
                   / SOLUTION DDFM=Satterthwaite OUTPM=Pred4a;
     RANDOM INTERCEPT WC_PvRF / GCORR TYPE=UN SUBJECT=schoolID; 
     ODS OUTPUT CovParms=Cov4a InfoCrit=Fit4a; * Save variances and fit; 
	ESTIMATE "Simple L2 PvRF Between Slope"  PvRF 1 CM_PvRF30 1;
     ESTIMATE "L2 PvRF Between*CM_PvRF30"     PvRF*CM_PvRF30 1 CM_PvRF30*CM_PvRF30 1;
	ESTIMATE "Simple L2 PRvF Between Slope"  PRvF 1 CM_PRvF19 1;
RUN; TITLE;
* Pseudo-R2 relative to random slope model (new baseline);
%PseudoR2(NCov=4, CovFewer=CovRand, CovMore1=Cov3b, CovMore2=Cov4a);
* Total-R2 relative to empty means model;
%TotalR2(DV=math, Pred1=Pred3b, Pred2=Pred4a);

TITLE "SAS Model 4b: Add Random Slope for Cluster-MC Paid/Reduced vs Free Lunch";
PROC MIXED DATA=work.Example4 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID;         
     MODEL math = PvRF CM_PvRF30 PvRF*CM_PvRF30 CM_PvRF30*CM_PvRF30 
                  PRvF CM_PRvF19 
                   / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT WC_PvRF WC_PRvF / GCORR TYPE=UN SUBJECT=schoolID; 
     ODS OUTPUT CovParms=CovRand2 InfoCrit=FitRand2; * Save variances and fit; 
RUN; TITLE;
* LRT for second random slope;
%FitTest(FitFewer=Fit4a, FitMore=FitRand2);

TITLE "SAS Model 5a: Add 2 Interaction Slopes for Paid/Reduced vs Free Lunch";
PROC MIXED DATA=work.Example4 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID;         
     MODEL math = PvRF CM_PvRF30 PvRF*CM_PvRF30 CM_PvRF30*CM_PvRF30 
                  PRvF CM_PRvF19 PRvF*CM_PRvF19 CM_PRvF19*CM_PRvF19 
                   / SOLUTION DDFM=Satterthwaite OUTPM=Pred5a;
     * Only PvRF random slope (PRvF is systematically varying);
     RANDOM INTERCEPT WC_PvRF / GCORR TYPE=UN SUBJECT=schoolID; 
     ODS OUTPUT CovParms=Cov5a; * Save variances; 
     ESTIMATE "Simple L2 PvRF Between"  PvRF 1 CM_PvRF30 1;
     ESTIMATE "L2 PvRF Between*CM_PvRF30"    PvRF*CM_PvRF30 1 CM_PvRF30*CM_PvRF30 1;
     ESTIMATE "Simple PRvF L2 Between PRvF"  PRvF 1 CM_PRvF19 1;
     ESTIMATE "L2 PRvF Between*CM_PRvF19"    PRvF*CM_PRvF19 1 CM_PRvF19*CM_PRvF19 1;
RUN; TITLE;
* Pseudo-R2 relative to single random slope model (same baseline);
%PseudoR2(NCov=4, CovFewer=CovRand, CovMore1=Cov3b, CovMore2=Cov5a);
* Total-R2 relative to empty means model (and change across predictors);
%TotalR2(DV=math, Pred1=Pred3b, Pred2=Pred5a);


***********************************************************************************;
* Close external file of saved results;
ODS RTF CLOSE;
